package com.yizhi.mall.application.excel.export;

import cn.hutool.core.date.DateUtil;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.write.metadata.style.WriteCellStyle;
import com.alibaba.excel.write.style.HorizontalCellStyleStrategy;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.core.application.context.TaskContext;
import com.yizhi.core.application.file.task.AbstractDefaultTask;
import com.yizhi.core.application.file.util.OssUpload;
import com.yizhi.mall.application.excel.model.TokenLogExcelModel;
import com.yizhi.mall.application.pojo.qo.ManageTokenLogQO;
import com.yizhi.mall.application.pojo.vo.ManageTokenLogVO;
import com.yizhi.mall.application.service.ITokenLogService;
import io.swagger.annotations.Api;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.poi.ss.usermodel.HorizontalAlignment;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.File;
import java.util.*;

@Api(tags = "消费记录导出接口")
@Slf4j
@Component
public class TokenLogExport extends AbstractDefaultTask<String, Map<String,Object>> {
    @Autowired
    private IdGenerator idGenerator;

    @Autowired
    private ITokenLogService tokenLogService;

    /**
     * 获取自定义表头
     *
     * @return
     */
    private List<List<String>> head() {
        List<String> secondHead = Arrays.asList("用户名", "姓名",
                "时间", "订单号", "类型", "备注", "消耗/充值数量");
        List<List<String>> list = new ArrayList<List<String>>();
        for (int i = 0; i < 7; i++) {
            List<String> head = new ArrayList<String>();
            head.add("兑换记录" + DateUtil.format(new Date(), "yyyyMMddHHmmss"));
            head.add(secondHead.get(i));
            list.add(head);
        }
        return list;
    }

    /**
     * 组装数据
     *
     * @return
     */
    private List<TokenLogExcelModel> getData(RequestContext context, ManageTokenLogQO qo) {
        List<TokenLogExcelModel> records = new ArrayList<>();

        try {
            qo.setPageSize(Integer.MAX_VALUE);
            List<ManageTokenLogVO> dataList = tokenLogService.getLogListByCondition(context, qo).getRecords();
            TokenLogExcelModel data = null;
            if (CollectionUtils.isNotEmpty(dataList)) {
                for (ManageTokenLogVO tokenLogVO : dataList) {
                    data = new TokenLogExcelModel();
                    BeanUtils.copyProperties(tokenLogVO, data);
                    records.add(data);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return records;

    }

    @Override
    protected String execute(Map<String,Object> map) {
        Date now = new Date();
        String formatDate = DateUtil.format(now, "yyyyMMddHHmmss");
        String serialNo = "TOKEN-LOG-EXPORT-" + formatDate;
        String taskName = "兑换记录明细-" + formatDate;
        RequestContext context =(RequestContext) map.get("context");
        Long accountId = context.getAccountId();
        Long siteId = context.getSiteId();
        Long companyId = context.getCompanyId();
        TaskContext taskContext = new TaskContext(idGenerator.generate(), serialNo, taskName, accountId
                , now, siteId, companyId);
        //记录任务日志
        working(taskContext);

        ManageTokenLogQO qo = (ManageTokenLogQO)map.get("qo");
        List<TokenLogExcelModel> excelModelList = getData(context, qo);
        if (CollectionUtils.isEmpty(excelModelList)) {
            log.info("没有当前需要导出的消费记录");
            fail(taskContext,"没有当前需要导出的消费记录！");
            return "fail";
        }

        log.info("开始组装消费记录");
        try {
            String fileName =  "兑换记录" + formatDate + ".xlsx";
            // 头的策略
            WriteCellStyle headWriteCellStyle = new WriteCellStyle();
            //设置表头左对齐
            headWriteCellStyle.setHorizontalAlignment(HorizontalAlignment.LEFT);
            // 内容的策略
            WriteCellStyle contentWriteCellStyle = new WriteCellStyle();
            //设置内容左对齐
            contentWriteCellStyle.setHorizontalAlignment(HorizontalAlignment.LEFT);


            String requestPath = "/manage/upload";//OssUpload.SAVE_PATH;
            File file = new File(requestPath);
            if (!file.exists()) {
                file.mkdirs();
            }
            String path = requestPath + File.separator + fileName;
            // 这个策略是 头是头的样式 内容是内容的样式 其他的策略可以自己实现
            HorizontalCellStyleStrategy horizontalCellStyleStrategy =
                    new HorizontalCellStyleStrategy(headWriteCellStyle, contentWriteCellStyle);

            EasyExcel.write(path, TokenLogExcelModel.class).registerWriteHandler(horizontalCellStyleStrategy).
                    head(head()).sheet("消费记录导出").doWrite(excelModelList);
            String upLoadUrl = OssUpload.upload(path, fileName);
            log.info("导出成功：upLoadUrl："+upLoadUrl);
            success(taskContext, "成功", upLoadUrl);
        } catch (Exception e) {
            fail(taskContext, "写入过程出现异常;");
            log.error("写入阿里云出错，出错信息：{}", e);
        }

        return taskName;
    }
}
