package com.yizhi.mall.application.handler;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.yizhi.core.application.cache.RedisCache;
import com.yizhi.system.application.feign.AuthFeignClient;
import com.yizhi.system.application.vo.practice.OrgAccountVo;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * @ClassName: AccountManagerHandler
 * @author: zjl
 * @date: 2021/1/15  14:09
 */
@Component
public class AccountManagerHandler {

    @Autowired
    private AuthFeignClient authFeignClient;
    @Autowired
    private RedisCache redisCache;

    private String REDIS_KEY_MANAGE = "MALL_ADMIN_MANAGE_ORG_OR_USER";
    // 缓存两小时
    private Long duration = 2 * 3600L;

    private Logger logger = LoggerFactory.getLogger(AccountManagerHandler.class);

    public OrgAccountVo getManageOrgAndUser(Long adminId, Long siteId) {
        String key = REDIS_KEY_MANAGE + adminId;
        Object cache = redisCache.get(key);
        if (cache != null) {
            logger.info("查询用户管辖区 缓存命中 :{}", adminId);
            return JSON.parseObject(cache.toString(), OrgAccountVo.class);
        } else {
            logger.info("查询用户管辖区 未命中缓存 :{}", adminId);
            OrgAccountVo vo = authFeignClient.getOrgAndUserOfAdmin(adminId, siteId);
            redisCache.set(key, JSON.toJSONString(vo), duration);
            return JSON.parseObject(JSONObject.toJSONString(vo), OrgAccountVo.class);
        }
    }
}
