package com.yizhi.mall.application.service.impl;

import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.mall.application.domain.TokenExchangeRule;
import com.yizhi.mall.application.domain.TokenExchangeRuleLog;
import com.yizhi.mall.application.pojo.qo.TokenExchangeRuleQO;
import com.yizhi.mall.application.pojo.vo.TokenExchangeRuleVO;
import com.yizhi.mall.application.repository.TokenExchangeRuleRepository;
import com.yizhi.mall.application.repository.TokenexchangeruleLogRepository;
import com.yizhi.mall.application.service.BaseService;
import com.yizhi.mall.application.service.ITokenExchangeRuleService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Objects;

/**
 * @ClassName: ITokenExchangeRuleServiceImpl
 * @author: zjl
 * @date: 2021/1/11  15:33
 */
@Service
public class ITokenExchangeRuleServiceImpl extends BaseService implements ITokenExchangeRuleService {

    @Autowired
    private TokenExchangeRuleRepository repository;

    @Autowired
    private TokenexchangeruleLogRepository logRepository;

    @Autowired
    private IdGenerator idGenerator;

    /**
     * 获取站点积分兑换金币规则
     * @return
     */
    @Override
    public TokenExchangeRule getTokenExchangeRule() {
        TokenExchangeRuleVO vo = new TokenExchangeRuleVO();
        RequestContext context = ContextHolder.get();
        Long companyId = context.getCompanyId();
        Long siteId = context.getSiteId();
        TokenExchangeRule rule = repository.findByCompanyIdAndSiteId(companyId, siteId);
        if (Objects.isNull(rule)) {
            return null;
        }
        return rule;
    }


    /**
     * 新增/修改站点积分兑换金币规则
     * @return
     * @param qo
     */
    @Override
    @Transactional
    public TokenExchangeRuleVO saveOrUpdateTokenExchangeRule(TokenExchangeRuleQO qo) {
        TokenExchangeRuleVO vo = new TokenExchangeRuleVO();
        RequestContext context = ContextHolder.get();
        Long companyId = context.getCompanyId();
        Long siteId = context.getSiteId();
        TokenExchangeRule get = repository.findByCompanyIdAndSiteId(companyId, siteId);
        if (Objects.isNull(get)) {
            //站点无兑换规则 新增
            get = new TokenExchangeRule();
            get.setId(idGenerator.generate());
        }
        get.setExchangeCondition(qo.getExchangeCondition());
        if (qo.getExchangeCondition()) {//条件限制
            get.setConditionRuleTokenCount(qo.getConditionRuleTokenCount());
            get.setConditionRuleType(qo.getConditionRuleType().toUpperCase());
        }
        get.setRate(qo.getRate());
        get.setTokenName(qo.getTokenName());
        TokenExchangeRule save = repository.save(get);
        synchronousToLog(save);
        return vo;
    }

    /**
     * 同步记录到log表
     * @param rule
     * @return
     */
    public void synchronousToLog(TokenExchangeRule rule) {
        //新增规则后 再新增记录到兑换规则log表
        TokenExchangeRuleLog log = new TokenExchangeRuleLog();
        log.setId(idGenerator.generate());
        log.setRuleId(rule.getId());
        log.setTokenName(rule.getTokenName());
        log.setRate(rule.getRate());
        log.setExchangeCondition(rule.getExchangeCondition());
        if (rule.getExchangeCondition()) {
            log.setConditionRuleTokenCount(rule.getConditionRuleTokenCount());
            log.setConditionRuleType(rule.getConditionRuleType().toUpperCase());
        }
        log =  logRepository.save(log);
    }
}
