package com.yizhi.mall.application.service.impl

import com.baomidou.mybatisplus.toolkit.CollectionUtils
import com.google.common.collect.Lists
import com.querydsl.core.types.Predicate
import com.yizhi.core.application.context.ContextHolder
import com.yizhi.core.application.context.RequestContext
import com.yizhi.mall.application.TokenLogType
import com.yizhi.mall.application.domain.QTokenLog
import com.yizhi.mall.application.domain.TokenLog
import com.yizhi.mall.application.handler.AccountManagerHandler
import com.yizhi.mall.application.pojo.qo.ManageTokenLogQO
import com.yizhi.mall.application.pojo.vo.ManageTokenLogVO
import com.yizhi.mall.application.pojo.vo.StudentTokenLogVO
import com.yizhi.mall.application.pojo.vo.TokenLogMonthTotalVO
import com.yizhi.mall.application.pojo.vo.TokenLogVO
import com.yizhi.mall.application.repository.TokenLogRepository
import com.yizhi.mall.application.service.BaseService
import com.yizhi.mall.application.service.ITokenLogService
import com.yizhi.system.application.vo.practice.AccountOrgVo
import com.yizhi.util.application.page.PageInfo
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.data.domain.Page
import org.springframework.data.domain.PageRequest
import org.springframework.data.querydsl.QSort
import org.springframework.stereotype.Service
import java.util.*
import java.util.stream.Collectors
import kotlin.collections.ArrayList
import kotlin.reflect.typeOf

/**
 *
 *
 * 服务实现类
 *
 *
 * @author MybatisCodeGenerator123
 * @since 2021-01-07
 */
@Service
class TokenLogServiceImpl : BaseService(), ITokenLogService {

    @Autowired
    lateinit var  tokenLogRepository: TokenLogRepository

    @Autowired
     lateinit var accountManagerhandler: AccountManagerHandler

    override fun getEnrollCount(relationId: Long, logType: Int):Long {
        val rc = ContextHolder.get()
        val qTokenLog = QTokenLog.tokenLog
        val predicate: Predicate = qTokenLog.companyId.eq(rc.companyId)
                .and(qTokenLog.siteId.eq(rc.siteId))
                .and(qTokenLog.relationId.eq(relationId))
                .and(qTokenLog.relationType.eq(logType))
                return jpaqueryFactory
                        .select(qTokenLog.accountId.countDistinct())
                        .from(qTokenLog)
                        .where(predicate)
                        .fetchCount() ?: 0
    }



    override fun getLogByRelationIdAndType(relationId: Long?, logType: Int?): List<TokenLog>? {
        val rc = ContextHolder.get()
        val qTokenLog = QTokenLog.tokenLog
        val predicate: Predicate = qTokenLog.accountId.eq(rc.accountId)
                .and(qTokenLog.companyId.eq(rc.companyId))
                .and(qTokenLog.siteId.eq(rc.siteId))
                .and(qTokenLog.relationId.eq(relationId))
                .and(qTokenLog.relationType.eq(logType))
        return jpaqueryFactory
                .select(qTokenLog)
                .from(qTokenLog)
                .where(predicate)
                .fetch()
    }

    override fun getOrderByOrderId(accountId: Long, companyId: Long, siteId: Long, orderId: String): TokenLog {
        val qTokenLog = QTokenLog.tokenLog
        val predicate = qTokenLog.companyId.eq(companyId)
                .and(qTokenLog.siteId.eq(siteId))
                .and(qTokenLog.accountId.eq(accountId))
                .and(qTokenLog.orderId.eq(orderId))
        return jpaqueryFactory
                .select(qTokenLog)
                .from(qTokenLog)
                .where(predicate)
                .fetchOne()!!
    }

    override fun getLogByAccountIdAndType(companyId: Long?, siteId: Long?, accountId: Long?, type: Int?, pageIndex: Int?, pageSize: Int?, startAt: Date?, endAt: Date?): Page<TokenLog> {
        val qTokenLog = QTokenLog.tokenLog
        var predicate = qTokenLog.companyId.eq(companyId)
                .and(qTokenLog.siteId.eq(siteId))
                .and(qTokenLog.accountId.eq(accountId))
                .and(qTokenLog.createdAt.goe(startAt))
                .and(qTokenLog.createdAt.lt(endAt))
        if (null != type) {
            predicate = predicate.and(qTokenLog.type.eq(type))
        }
        val page = PageRequest.of(pageIndex!! - 1, pageSize!!, QSort.by(qTokenLog.createdAt.desc()))
        return tokenLogRepository!!.findAll(predicate, page)
    }

    override fun getLastLog(type: Int?, companyId: Long?, siteId: Long?, accountId: Long?): TokenLog? {
        val qTokenLog = QTokenLog.tokenLog
        var predicate = qTokenLog.accountId.eq(accountId)
                .and(qTokenLog.companyId.eq(companyId))
                .and(qTokenLog.siteId.eq(siteId))
        if (null != type) {
            predicate = predicate.and(qTokenLog.type.eq(type))
        }
        return jpaqueryFactory
                .select(qTokenLog)
                .from(qTokenLog)
                .where(predicate)
                .orderBy(qTokenLog.createdAt.desc())
                .limit(1)
                .fetchOne()
    }

    override fun getTokenTotal(startAt: Date, endAt: Date,type: Int?): TokenLogMonthTotalVO {
        val rc = ContextHolder.get()
        val qTokenLog = QTokenLog.tokenLog
        var predicate = qTokenLog.accountId.eq(rc.accountId)
                .and(qTokenLog.companyId.eq(rc.companyId))
                .and(qTokenLog.siteId.eq(rc.siteId))
                .and(qTokenLog.createdAt.goe(startAt))
                .and(qTokenLog.createdAt.lt(endAt))
        if (type != null) {
            predicate = predicate.and(qTokenLog.type.eq(type))
        }

        val list = jpaqueryFactory.select(
                qTokenLog.changeCount.sum(),
                qTokenLog.changeType)
                .from(qTokenLog)
                .where(predicate)
                .groupBy(qTokenLog.changeType)
                .fetch()
        val vo = TokenLogMonthTotalVO()


        if (list == null || list.isEmpty()) {
            return vo
        }
        for (tuple in list) {
            if (null == tuple.get(qTokenLog.changeType)) {
                return vo
            }
            if (tuple.get(qTokenLog.changeType) == 1) {
                vo.receiveCount = tuple.get(qTokenLog.changeCount.sum())
            }
            if (tuple.get(qTokenLog.changeType) == -1) {
                vo.useCount = tuple.get(qTokenLog.changeCount.sum())
            }
        }
        return vo
    }

    /**
     * 获取指定用户消费记录
     * @param companyId
     * @param siteId
     * @param accountId
     * @param type
     * @return
     */
    override fun getLogListByAccountIdAndType(companyId: Long?, siteId: Long?, accountId: Long?, type: Int?): List<TokenLog>? {
        val qTokenLog = QTokenLog.tokenLog
        var predicate = qTokenLog.companyId.eq(companyId)
                .and(qTokenLog.siteId.eq(siteId))
                .and(qTokenLog.accountId.eq(accountId))
        if (null != type) {
            predicate = predicate.and(qTokenLog.type.eq(type))
        }
        return Lists.newArrayList(tokenLogRepository.findAll(predicate).iterator())
    }

    /**
     * 管理端获取消费记录
     * @param qo
     * @return
     */
    override fun getLogListByCondition(context: RequestContext?, qo: ManageTokenLogQO?): PageInfo<ManageTokenLogVO>? {
        val adminId = context!!.accountId
        val siteId = context.siteId
        val pageNo = qo!!.pageNo
        val pageSize = qo.pageSize
        var kwd = qo.kwd.trim { it <= ' ' }
        val voPageInfo = PageInfo<ManageTokenLogVO>()
        voPageInfo.pageNo = qo.pageNo
        voPageInfo.pageSize = qo.pageSize
        val qTokenLog = QTokenLog.tokenLog
        var predicate = qTokenLog.companyId.eq(context.companyId)
                .and(qTokenLog.siteId.eq(context.siteId))
        if (qo.startAt != null && qo.endAt != null) {
            predicate = predicate.and(qTokenLog.createdAt.goe(qo.startAt))
                    .and(qTokenLog.createdAt.lt(qo.endAt))
        }
        if (null != qo.type) {
            predicate = predicate.and(qTokenLog.type.eq(qo.type))
        }
        if (null != kwd && kwd !== "") {
            kwd = "%$kwd%"
            predicate = predicate.and(qTokenLog.accountFullName.like(kwd)
                    .or(qTokenLog.accountName.like(kwd))
                    .or(qTokenLog.orderId.like(kwd)))
        }
        if (!context.isAdmin) {
            //不是管理员 查询管辖区
            val vo = accountManagerhandler!!.getManageOrgAndUser(adminId, siteId)
            val accountIds = vo.users.stream().map { obj: AccountOrgVo -> obj.accountId }.collect(Collectors.toList())
            predicate = predicate.and(qTokenLog.orgId.`in`(vo.orgs)
                    .or(qTokenLog.accountId.`in`(accountIds)))
        }
        val page = PageRequest.of(pageNo - 1, pageSize, QSort.by(qTokenLog.createdAt.desc()))
        val tokenLogPage = tokenLogRepository!!.findAll(predicate, page)
        val content = tokenLogPage.content
        if (CollectionUtils.isNotEmpty(content)) {
            var voList = transform(content)
            voPageInfo.pageTotal = tokenLogPage.totalPages
            voPageInfo.pageRecords = tokenLogPage.totalElements.toInt()
            voPageInfo.setRecords(voList)
        }
        return voPageInfo
    }

    /**
     * 转换成volist
     * @param tokenLogList
     * @return
     */
    fun transform(tokenLogList: List<TokenLog?>): List<ManageTokenLogVO> {
        val voList = ArrayList<ManageTokenLogVO>(tokenLogList.size)
        var vo: ManageTokenLogVO? = null
        for (tokenLog in tokenLogList) {
            vo = ManageTokenLogVO()
            vo.createdAt = tokenLog!!.createdAt
            vo.name = tokenLog.accountName
            vo.fullName = if (null == tokenLog.accountFullName) tokenLog.accountName else tokenLog.accountFullName
            vo.orderId = tokenLog.orderId
            vo.type = tokenLog.type
            vo.remark = tokenLog.relationName
            vo.relationName = tokenLog.relationName
            vo.count = tokenLog.changeCount
            voList.add(vo)
        }
        return voList
    }

    /**
     * 获取学员端兑换记录分页
     * @param pageNo
     * @param pageSize
     * @return
     */
    override fun getLogPageByUserId(pageNo: Int?, pageSize: Int?): PageInfo<StudentTokenLogVO>? {
        var pageNo = pageNo
        var pageSize = pageSize
        val voPageInfo = PageInfo<StudentTokenLogVO>()
        pageNo = pageNo ?: 1
        pageSize = pageSize ?: 10
        voPageInfo.pageNo = pageNo
        voPageInfo.pageSize = pageSize
        val page = getLogListByUserId(pageNo, pageSize)
        val content = page.content;
        if (CollectionUtils.isNotEmpty(content)) {
            var voList = transformStudentTokenLogVO(content)
            voPageInfo.pageTotal = page.totalPages
            voPageInfo.pageRecords = page.totalElements.toInt()
            voPageInfo.records = voList
        }
        return voPageInfo
    }

    /**
     * 获取学员端兑换记录分页后的list
     * @param pageNo
     * @param pageSize
     * @return
     */
    fun getLogListByUserId(pageNo: Int, pageSize: Int?): Page<TokenLog> {
        val context = ContextHolder.get()
        val userId = context.accountId
        val companyId = context.companyId
        val siteId = context.siteId
        val qTokenLog = QTokenLog.tokenLog
        val predicate = qTokenLog.companyId.eq(companyId)
                .and(qTokenLog.siteId.eq(siteId))
                .and(qTokenLog.accountId.eq(userId)
                        .and(qTokenLog.changeType.eq(-1)) //-1为兑换
                )
        val page = PageRequest.of(pageNo - 1, pageSize!!, QSort.by(qTokenLog.createdAt.desc()))
        val tokenLogPage = tokenLogRepository!!.findAll(predicate, page)
        return tokenLogPage;
    }

    /**
     * 转换成StudentTokenLogVOlist
     * @param tokenLogList
     * @return
     */
    fun transformStudentTokenLogVO(tokenLogList: List<TokenLog>): List<StudentTokenLogVO> {
        val voList = ArrayList<StudentTokenLogVO>(tokenLogList.size)
        var vo: StudentTokenLogVO? = null
        for (tokenLog in tokenLogList) {
            vo = StudentTokenLogVO()
            vo.createdAt = tokenLog!!.createdAt
            vo.relationIconUrl = tokenLog.relationIconUrl
            vo.type = tokenLog.type
            vo.relationName = tokenLog.relationName
            vo.relationId = tokenLog.relationId
            vo.count = tokenLog.changeCount
            voList.add(vo)
        }
        return voList
    }
}