package com.yizhi.mall.application.utils

import cn.hutool.core.date.DateUtil
import com.yizhi.mall.application.ExchangeConditionType
import java.util.*

object MallDateUtils {

    /**
     * @param conditionType 时间段枚举
     * @param date 时间点
     * @return 返回指定时间 默认的时间范围的开始 结束时间
     */
    fun getSAtAndEAtByConditionCode(conditionType: ExchangeConditionType, date: Date): Pair<Date, Date> {
        var calendar = Calendar.getInstance()
        calendar.time = date
        calendar.set(Calendar.HOUR_OF_DAY,0)
        calendar.set(Calendar.MINUTE,0)
        calendar.set(Calendar.SECOND,0)

        var startAt:Date ? = null
        var endAt :Date ? = null
        when(conditionType.code) {
            // 年
            ExchangeConditionType.YEAR.code -> {
                calendar.set(Calendar.MONTH,Calendar.JANUARY)
                calendar.set(Calendar.DAY_OF_MONTH,1)
                startAt = calendar.time
                calendar.add(Calendar.YEAR ,1)
                endAt = calendar.time
            }

            // 半年
            ExchangeConditionType.HALFYEAR.code -> {
                calendar.set(Calendar.MONTH,Calendar.JULY)
                calendar.set(Calendar.DAY_OF_MONTH,1)
                // 下半年
                if (date.time  >= calendar.time.time) {
                    startAt = calendar.time
                    calendar.add(Calendar.MONTH ,6)
                    endAt = calendar.time
                }else {
                    endAt = calendar.time
                    calendar.set(Calendar.MONTH,Calendar.JANUARY)
                    startAt = calendar.time
                }
            }

            // 季度
            ExchangeConditionType.QUARTER.code -> {
                val quarter = DateUtil.quarter(calendar.time) - 1
                calendar.set(Calendar.MONTH,Calendar.JANUARY)
                calendar.set(Calendar.DAY_OF_MONTH,1)
                calendar.add(Calendar.MONTH,quarter*3)
                startAt = calendar.time
                calendar.add(Calendar.MONTH ,3)
                endAt = calendar.time
            }

            // 月
            ExchangeConditionType.MONTH.code -> {
                calendar.set(Calendar.MONTH,calendar.get(Calendar.MONTH))
                calendar.set(Calendar.DAY_OF_MONTH,1)
                startAt = calendar.time
                calendar.add(Calendar.MONTH ,1)
                endAt = calendar.time
            }

            // 周
            ExchangeConditionType.WEEK.code -> {
                calendar.set(Calendar.DAY_OF_WEEK,Calendar.MONDAY)
                calendar.set(Calendar.WEEK_OF_YEAR,calendar.get(Calendar.WEEK_OF_YEAR))
                startAt = calendar.time
                calendar.add(Calendar.WEEK_OF_YEAR ,1)
                endAt = calendar.time

            }

            // 天
            ExchangeConditionType.DAY.code -> {
                startAt = calendar.time
                calendar.add(Calendar.DAY_OF_YEAR ,1)
                endAt = calendar.time
            }
        }
        return Pair(startAt!!,endAt!!);
    }


}