package com.yizhi.mall.application.xservice.impl

import com.yizhi.application.orm.id.IdGenerator
import com.yizhi.core.application.cache.RedisCache
import com.yizhi.core.application.context.ContextHolder
import com.yizhi.core.application.exception.BizException
import com.yizhi.mall.application.TokenLogType
import com.yizhi.mall.application.config.MallMsgEnum
import com.yizhi.mall.application.config.RedisLock
import com.yizhi.mall.application.config.constant.TokenLogRelationDto
import com.yizhi.mall.application.domain.*
import com.yizhi.mall.application.repository.AccountExchangeLogRepository
import com.yizhi.mall.application.repository.AccountExchangeSumRepository
import com.yizhi.mall.application.repository.AccountTokenRepository
import com.yizhi.mall.application.repository.TokenLogRepository
import com.yizhi.mall.application.service.*
import com.yizhi.mall.application.utils.MallOrderIdUtils
import com.yizhi.mall.application.xservice.AccountTokenXService
import com.yizhi.mall.application.xservice.TokenConfigXService
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.stereotype.Service
import javax.transaction.Transactional


/**
 * @Date 2021/1/18 1:32 下午
 * @Author lvjianhui
 **/
@Service
open class AccountTokenXServiceImpl : AccountTokenXService {

    @Autowired
    lateinit var iAccountExchangeSumService: IAccountExchangeSumService

    @Autowired
    lateinit var tokenConfigXService: TokenConfigXService

    @Autowired
    lateinit var accountTokenService: IAccountTokenService

    @Autowired
    lateinit var accountTokenRepository: AccountTokenRepository

    @Autowired
    lateinit var tokenLogRepository: TokenLogRepository

    @Autowired
    lateinit var accountExchangeLogRepository: AccountExchangeLogRepository

    @Autowired
    lateinit var idGenerator: IdGenerator

    @Autowired
    lateinit var accountExchangeSunRepository: AccountExchangeSumRepository

    @Autowired
    lateinit var redisCache: RedisCache

    @Transactional
    override fun updateTokenAccount(changeTokenAbs: Int, logType: TokenLogType, rule: TokenExchangeRule?, relationMsg: TokenLogRelationDto?) {

        val changeToken = changeTokenAbs * logType.addOrsSub
        val rc = ContextHolder.get()
        val redisAccountKey = RedisLock.ACCOUNT_TOKEN_LOCK.format(rc.accountId)
        val timeout = 10L
        try {
            // 增加分布式锁。如果set完成，没有执行 expire 和 delete方法，则会出现key永不过期的问题
            val lock = redisCache.setIfAbsent(redisAccountKey, "1")
            redisCache.expire(redisAccountKey, timeout)
            if (!lock) {
                throw BizException(MallMsgEnum.ACCOUNT_TOKEN_PROCESSING.code, MallMsgEnum.ACCOUNT_TOKEN_PROCESSING.msg)
            }
            // 增加金币
            var accountToken = accountTokenService.tokenAccount
            val beforToken = accountToken?.amount ?: 0
            if (null == accountToken) {
                accountToken = AccountToken()
                accountToken.id = idGenerator.generate()
                accountToken.amount = changeToken
            } else {
                accountToken.amount = accountToken.amount + changeToken
            }
            accountTokenRepository.save(accountToken)

            val log = TokenLog()
            log.id = idGenerator.generate()
            log.changeType = logType.addOrsSub
            log.changeCount = changeToken
            log.beforeCount = beforToken
            log.afterCount = log.changeCount + log.beforeCount
            log.type = logType.code
            log.relationIconUrl = relationMsg?.relationIconUrl
            log.relationName = relationMsg?.relationName
            log.relationId = relationMsg?.relationId
            log.relationType = logType.code
            if (logType.code == TokenLogType.POINT_EXCHANGE.code) {
                log.costCount = rule!!.rate.toInt() * changeToken
                log.relationName = "-${log.costCount}积分"
            }
            log.msg = logType.msg
            log.orderId = logType.orderPre + MallOrderIdUtils.generateorderIdByUserAndLog(log.id)
            tokenLogRepository.save(log)
            if (logType.addOrsSub == -1 || null == rule) {
                return
            }

            val type = TokenLogType.POINT_EXCHANGE
            var sum = iAccountExchangeSumService.getSumByType(type.code)
            if (sum == null) {
                sum = AccountExchangeSum()
                sum.id = idGenerator.generate()
                sum.type = type.code
                sum.total = rule.rate.toInt() * changeToken
                sum.tokenTotal = changeToken
            } else {
                sum.total = sum.total + rule.rate.toInt() * changeToken
                sum.tokenTotal = sum.tokenTotal + changeToken
            }
            accountExchangeSunRepository.save(sum)

            val exchangeLog = AccountExchangeLog()
            exchangeLog.id = idGenerator.generate()
            exchangeLog.count = rule.rate.toInt() * changeToken
            exchangeLog.exchangeRate = rule.rate
            exchangeLog.tokenCount = changeToken
            exchangeLog.type = TokenLogType.POINT_EXCHANGE.code
            accountExchangeLogRepository.save(exchangeLog)
            redisCache.delete(redisAccountKey)
        } catch (e: Exception) {
            redisCache.delete(redisAccountKey)
        } finally {
            redisCache.delete(redisAccountKey)
        }
    }

}