package com.yizhi.mall.application.xservice.impl;

import com.baomidou.mybatisplus.toolkit.CollectionUtils;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.mall.application.TokenLogType;
import com.yizhi.mall.application.domain.TokenLog;
import com.yizhi.mall.application.pojo.vo.QualityContentVO;
import com.yizhi.mall.application.repository.TokenLogRepository;
import com.yizhi.mall.application.service.IAccountTokenService;
import com.yizhi.mall.application.service.ITokenLogService;
import com.yizhi.mall.application.xservice.QualityContentXService;
import com.yizhi.system.application.system.remote.AccountClient;
import com.yizhi.training.application.feign.PaidTrainingProjectClient;
import com.yizhi.training.application.param.PaidTrainingProjectQO;
import com.yizhi.training.application.vo.api.PaidTrainingProjectVO;
import com.yizhi.util.application.page.PageInfo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.stream.Collectors;

/**
 * @ClassName: QualityContentXServiceImpl
 * @author: zjl
 * @date: 2021/1/12  9:51
 */
@Service
public class QualityContentXServiceImpl implements QualityContentXService {

    @Autowired
    private AccountClient accountClient;

    @Autowired
    private PaidTrainingProjectClient paidTrainingProjectClient;

    @Autowired
    private IAccountTokenService accountTokenService;

    @Autowired
    private TokenLogRepository tokenLogRepository;

    @Autowired
    private ITokenLogService tokenLogService;

    /**
     * 获取精品内容
     * @return
     */
    @Override
    public QualityContentVO getQualityContent() {
        RequestContext context = ContextHolder.get();
        Long companyId = context.getCompanyId();
        Long siteId = context.getSiteId();
        Long accountId = context.getAccountId();
        String fullName = context.getAccountFullName() ==null ? context.getAccountName() : context.getAccountFullName();
        Integer tokenCount = accountTokenService.getTokenCountById(companyId, siteId, accountId);
        Integer exchangeLogsCount = tokenLogRepository.countAccountExchangeLogsByAccountIdAndSiteIdAndChangeType(accountId, siteId, -1);
        QualityContentVO vo = new QualityContentVO(accountId, fullName, context.getAccountName(), context.getHeadPortrait(), tokenCount, exchangeLogsCount);
        return vo;
    }

    /**
     * 获取付费项目并排序筛选
     * @param qo
     * @return
     */
    @Override
    public PageInfo<PaidTrainingProjectVO> getPaidTrainingProject(PaidTrainingProjectQO qo) {
        PageInfo<PaidTrainingProjectVO> voPageInfo = new PageInfo<>();
        Integer pageNo = qo.getPageNo();
        Integer pageSize = qo.getPageSize();
        voPageInfo.setPageNo(pageNo);
        voPageInfo.setPageSize(pageSize);
        List<PaidTrainingProjectVO> voList = null;

        RequestContext context = ContextHolder.get();
        Long companyId = context.getCompanyId();
        Long siteId = context.getSiteId();
        Long accountId = context.getAccountId();
        qo.setSiteId(siteId);
        //用户金币数
        Integer tokenCount = accountTokenService.getTokenCountById(companyId, siteId, accountId);
        //获取排序后的付费项目
        voList = paidTrainingProjectClient.getPaidTrainingProject(qo);
        if (CollectionUtils.isNotEmpty(voList)) {
            //筛选
            if (qo.getExchangeState() > 0) {
                //用户兑换项目记录
                List<TokenLog> hasExchangeProjects = tokenLogService.getLogListByAccountIdAndType(companyId, siteId, accountId, TokenLogType.PROJECT_APPLY.getCode());
                List<Long> hasExchangeIds = hasExchangeProjects.stream().map(TokenLog::getRelationId).collect(Collectors.toList());
                voList = filterList(hasExchangeIds, voList, tokenCount, qo.getExchangeState());
            }
            voPageInfo.setPageTotal( (voList.size()  +  pageSize  - 1) / pageSize);
            voPageInfo.setPageRecords(voList.size());
            Integer start = (pageNo - 1) * pageSize;
            Integer end = pageNo * pageSize > voList.size() ? voList.size() : pageNo * pageSize;
            voList = voList.subList(start, end);
        }
        voPageInfo.setRecords(voList);
        voPageInfo.setPageSortField(qo.getOrderField());
        voPageInfo.setPageSortType(qo.getOrder());
        return voPageInfo;
    }

    /**
     * 通过兑换状态筛选
     */
    public List<PaidTrainingProjectVO> filterList(List<Long> hasExchangeIds, List<PaidTrainingProjectVO> voList, Integer tokenCount, Integer exchangeState) {
        switch (exchangeState) {
            //1(已兑换)
            case 1:
                voList = voList.stream().filter(t -> hasExchangeIds.contains(t.getId())).collect(Collectors.toList());
                break;
            //2(未兑换)
            case 2:
                voList = voList.stream().filter(t -> !hasExchangeIds.contains(t.getId())).collect(Collectors.toList());
                break;
            //3(可兑换: 未兑换 && 金币数>=实际价格)
            case 3:
                voList = voList.stream().filter(t -> (!hasExchangeIds.contains(t.getId())) && (tokenCount.compareTo(t.getActualPrice()) >=0) ).collect(Collectors.toList());
                break;
            default:
                break;
        }
        return voList;
    }
}
