package com.yizhi.mall.application.xservice.impl

import com.yizhi.core.application.context.ContextHolder
import com.yizhi.core.application.context.RequestContext
import com.yizhi.mall.application.ExchangeConditionType
import com.yizhi.mall.application.domain.TokenLog
import com.yizhi.mall.application.pojo.vo.OrderDetailVO
import com.yizhi.mall.application.pojo.vo.TokenLogMonthTotalVO
import com.yizhi.mall.application.pojo.vo.TokenLogVO
import com.yizhi.mall.application.service.ITokenLogService
import com.yizhi.mall.application.utils.MallDateUtils
import com.yizhi.mall.application.xservice.TokenLogXService
import com.yizhi.util.application.page.PageInfo
import org.apache.commons.lang.StringUtils
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.stereotype.Service
import java.util.*


/**
 * @Date 2021/1/7 10:54 上午
 * @Author lvjianhui
 **/
@Service
class TokenLogXServiceImpl : TokenLogXService {

    @Autowired
    lateinit var tokenLogService: ITokenLogService

    override fun getLogByType(startAt: Date, endAt: Date, type: Int?, pageIndex: Int, pageSize: Int): PageInfo<TokenLogVO> {
        val context = ContextHolder.get()
        val pageInfo = PageInfo<TokenLogVO>()
        pageInfo.pageNo = pageIndex
        pageInfo.pageSize = pageSize
        val page = tokenLogService.getLogByAccountIdAndType(
                context.companyId,
                context.siteId,
                context.accountId,
                type,
                pageIndex,
                pageSize,
                startAt,
                endAt)
        pageInfo.pageTotal = page.totalPages
        val list = page.content
        val vos = arrayListOf<TokenLogVO>()
        list.forEach {
            val vo = TokenLogVO()
            vo.createdAt = it.createdAt
            vo.type = it.type
            vo.orderId = it.orderId
            vo.relationId = it.relationId
            vo.relationName = it.relationName
            vo.count = it.changeCount
            vos.add(vo)
        }
        pageInfo.pageRecords = page.totalElements.toInt()
        pageInfo.records = vos
        return pageInfo
    }

    override fun getTokenTotal(startAt: Long?, type: Int?, pageIndex: Int, pageSize: Int): TokenLogMonthTotalVO? {
        var vo = TokenLogMonthTotalVO()
        val pageInfo = PageInfo<TokenLogVO>()
        pageInfo.pageNo = pageIndex
        pageInfo.pageSize = pageSize
        pageInfo.pageTotal = 0
        pageInfo.pageRecords = 0
        vo.items = pageInfo
        val currentDate = getStartAndEndAt(startAt, type) ?: let {
            // 没有任何记录
            vo.timestamp = Date().time
            return vo
        }

        val startAndEnd = MallDateUtils.getSAtAndEAtByConditionCode(ExchangeConditionType.MONTH, currentDate)
        val startAt = startAndEnd.first
        val endAt = startAndEnd.second
        vo = tokenLogService.getTokenTotal(startAt, endAt, type)
        vo.items = getLogByType(startAt, endAt, type, pageIndex, pageSize)
        vo.timestamp = currentDate.time
        return vo
    }

    override fun getOrderDetail(accountId: Long, companyId: Long, siteId: Long, orderId: String): OrderDetailVO {
        val log = tokenLogService.getOrderByOrderId(accountId, companyId, siteId, orderId)!!
        val vo = OrderDetailVO()
        vo.tokenCount = log.changeCount
        vo.costCount = log.costCount
        vo.type = log.type
        vo.createdAt = log.createdAt
        vo.accountName = log.accountFullName
        if (StringUtils.isEmpty(vo.accountName)) {
            vo.accountName = log.accountName
        }
        vo.orderId = log.orderId
        return vo
    }

    /**
     * 按月获取开始结束
     */
    private fun getStartAndEndAt(startAt: Long?, type: Int?): Date? {
        val context = ContextHolder.get()
        if (startAt != null) {
            return Date(startAt)
        }
        // 时间为null，查询最后一条log，获取最新一条记录的月份
        val tokenLog: TokenLog = tokenLogService.getLastLog(type, context.companyId, context.siteId, context.accountId)
                ?: return null
        return tokenLog.createdAt
    }
}