package com.yizhi.mall.application.xservice.impl

import com.yizhi.core.application.context.ContextHolder
import com.yizhi.core.application.exception.BizException
import com.yizhi.enroll.application.feign.EnrollFeignClient
import com.yizhi.mall.application.ExchangeConditionType
import com.yizhi.mall.application.TokenLogType
import com.yizhi.mall.application.TokenSwitchType
import com.yizhi.mall.application.config.MallMsgEnum
import com.yizhi.mall.application.config.constant.TokenLogRelationDto
import com.yizhi.mall.application.domain.*
import com.yizhi.mall.application.pojo.qo.ApplyProjectPO
import com.yizhi.mall.application.pojo.qo.TokenExchangePO
import com.yizhi.mall.application.service.*
import com.yizhi.mall.application.utils.MallDateUtils
import com.yizhi.mall.application.xservice.AccountTokenXService
import com.yizhi.mall.application.xservice.TokenConfigXService
import com.yizhi.mall.application.xservice.TokenXService
import com.yizhi.mall.application.xservice.TrXService
import com.yizhi.point.application.feign.PointDetailsFeignClients
import com.yizhi.training.application.feign.TrainingProjectClient
import org.slf4j.LoggerFactory
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.stereotype.Service
import java.math.BigDecimal
import java.util.*


/**
 * @Date 2021/1/13 2:34 下午
 * @Author lvjianhui
 **/
@Service
open class TokenXServiceImpl : TokenXService {

    private val logger = LoggerFactory.getLogger(javaClass)

    @Autowired
    lateinit var pointClient: PointDetailsFeignClients

    @Autowired
    lateinit var iAccountExchangeSumService: IAccountExchangeSumService

    @Autowired
    lateinit var tokenConfigXService: TokenConfigXService

    @Autowired
    lateinit var iAccountExchangeLogService: IAccountExchangeLogService

    @Autowired
    lateinit var tokenExchangeRuleService: ITokenExchangeRuleService

    @Autowired
    lateinit var accountTokenService: IAccountTokenService

    @Autowired
    lateinit var accountTokenXService: AccountTokenXService

    @Autowired
    lateinit var tokenLogService: ITokenLogService

    @Autowired
    lateinit var trainingProjectClient: TrainingProjectClient
    @Autowired
    lateinit var enrollFeignClient: EnrollFeignClient
    @Autowired
    lateinit var trXService: TrXService

    override fun costToProject(po: ApplyProjectPO) {

        var msg = TokenLogRelationDto()
        when(po.relationType) {
            TokenLogType.PROJECT_APPLY.code -> {
                msg = projectApplyCheck(po,msg)
            }
        }
        val accountToken = accountTokenService.tokenAccount
        val userToken = accountToken?.amount ?: 0
        if (msg.needToken > userToken) {
            // 余额不足
            throw BizException(MallMsgEnum.TOKEN_AMOUNT_ERROR.code, MallMsgEnum.TOKEN_AMOUNT_ERROR.msg)
        }
        trXService.trApplyCostToken(msg.needToken, TokenLogType.PROJECT_APPLY, null, msg)
    }

    private fun projectApplyCheck(po: ApplyProjectPO,msg:TokenLogRelationDto):TokenLogRelationDto {
        val costList = tokenLogService.getLogByRelationIdAndType(po.relationtId, po.relationType)
        if (costList != null && costList.size > 0) {
            // 已报名
            throw BizException(MallMsgEnum.PROJECT_APPLYED.code, MallMsgEnum.PROJECT_APPLYED.msg)
        }
        val vo = trainingProjectClient.getOne(po.relationtId)
        // 付费
        if (vo == null || vo.status != 1 || vo.enableEnroll != 1) {
            throw BizException(MallMsgEnum.PROJECT_PARAMS_ERROR.code, MallMsgEnum.PROJECT_PARAMS_ERROR.msg)
        }
        // 报名条件
        val enrool = enrollFeignClient.selectByProjectId(po.relationtId)
        if (enrool == null || enrool.enablePay != 1) {
            throw BizException(MallMsgEnum.PROJECT_PARAMS_ERROR.code, MallMsgEnum.PROJECT_PARAMS_ERROR.msg)
        }

        val currentTimeMillis = System.currentTimeMillis()

        if((enrool.startTime != null && enrool.startTime.time > currentTimeMillis)
                || (enrool.endTime != null && enrool.endTime.time < currentTimeMillis)
        ) {
            throw BizException(MallMsgEnum.PROJECT_ENROOL_TIME_ERROR.code, MallMsgEnum.PROJECT_ENROOL_TIME_ERROR.msg)
        }


        // 报名人数
        if (enrool.personLimitNum != null && enrool.personLimitNum != 0) {
            // 查询报名人数
            val count = tokenLogService.getEnrollCount(po.relationtId, po.relationType)
            if (count >= enrool.personLimitNum) {
                val msg = MallMsgEnum.PROJECT_ENROLL_ACCOUNT_NUM_LIMIT
                throw BizException(msg.code, msg.msg)
            }
        }
        msg.relationIconUrl = vo.logoImg
        msg.relationId = po.relationtId
        msg.relationName = vo.name
        msg.needToken = enrool.actualPrice ?: 0
        return msg
    }

    override fun getPointCount(): Int {
        val rc = ContextHolder.get()
        // 积分总和
        val pointCount = pointClient.queryPiont(rc.accountId, rc.companyId, rc.siteId, rc.orgId) ?: 0
        val pointUsedCount = iAccountExchangeSumService.getCostPointTotal()
        if (pointUsedCount > pointCount) {
            return 0
        }
        return pointCount - pointUsedCount
    }

    override fun pointExchangeToken(po: TokenExchangePO) {
        // 查看开关
        checkExchangeSwitch()
        val needExchangeToken = po.tokenCount
        val rule = tokenExchangeRuleService.tokenExchangeRule ?: let {
            logger.error("兑换规则配置异常")
            throw BizException(MallMsgEnum.POINT_RULE_ERROR.code, MallMsgEnum.POINT_RULE_ERROR.msg)
        }
        // 查看规则
        checkRuleLimit(rule, needExchangeToken)
        accountTokenXService.updateTokenAccount(needExchangeToken, TokenLogType.POINT_EXCHANGE, rule, null)
    }

    private fun checkRuleLimit(rule: TokenExchangeRule, needExchangeToken: Int) {
        if (needExchangeToken <= 0) {
            val msg = MallMsgEnum.EXCHANGE_TOKEN_AMOUNT_ERROR
            throw BizException(msg.code, msg.msg)
        }

        val ruleRate = rule.rate

        // 积分够不够
        val canUsePointCount = getPointCount()
        val needPointCount = ruleRate * BigDecimal(needExchangeToken)
        // 需要积分大于可用积分
        if (needPointCount.compareTo(BigDecimal(canUsePointCount)) == 1) {
            val msg = MallMsgEnum.POINT_AMOUNT_NOT_ENOUGH
            throw BizException(msg.code, msg.msg)
        }

        if (!rule.exchangeCondition) {
            return
        }
        val ruleType = rule.conditionRuleType
        val ruleCount = rule.conditionRuleTokenCount
        val currentTime = Date()
        var startAndEndAt: Pair<Date, Date>? = null
        if (ruleType != ExchangeConditionType.ALLTIME.code) {
            startAndEndAt = MallDateUtils.getSAtAndEAtByConditionCode(ExchangeConditionType.getByCode(ruleType), currentTime)
        }
        // 查询已兑换金币数
        val currentDateToken = iAccountExchangeLogService.getPointExchangeTokenTotalByDate(startAndEndAt?.first, startAndEndAt?.second)
        val afterCount = needExchangeToken + currentDateToken
        if (afterCount > ruleCount) {
            val msg = MallMsgEnum.POINT_EXCHANGE_TOKEN_LIMIT
            throw BizException(msg.code, msg.msg)
        }

    }

    private fun checkExchangeSwitch() {
        val vo = tokenConfigXService.getSwitch(TokenSwitchType.POINT_EXCHANGE.type)
        if (null == vo || vo.isEmpty() || !vo.first().tokenSwitch) {
            val code = MallMsgEnum.POINT_EXCHANGE_CLOSE
            throw BizException(code.code, code.msg)
        }
    }
}